var mediawrapUtils = {  
  observe: function(subject, topic, data) {
    this.statusChanged();
  },

  register: function() {
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService);
    observerSvc.addObserver(this, "mediawrapStatusChanged", false);
  },

  unregister: function() {
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService);
    observerSvc.removeObserver(this, "mediawrapStatusChanged");
  },

  updateStatusIcon: function() {
    var strBundle = document.getElementById("mediawrap-Strings");
    var statusImage = document.getElementById("mediawrap-status-image");
    if (!statusImage) return;
    var statusEnabledItem = document.getElementById("mediawrap-status-enabled-item");
    if ( this.getEnabled() ) {
      statusImage.src = "chrome://mediawrap_images/content/status_on.png";
      statusImage.tooltipText = strBundle.getString("Enabled");
      statusEnabledItem.setAttribute("checked", true);
    }
    else {
      statusImage.src = "chrome://mediawrap_images/content/status_off.png";
      statusImage.tooltipText = strBundle.getString("Disabled");
      statusEnabledItem.setAttribute("checked", false);
    }
  },

  statusChanged: function() {
    // Only use the new stylesheet api
    var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"]
          .getService(Components.interfaces.nsIStyleSheetService);
    var ios = Components.classes["@mozilla.org/network/io-service;1"]
          .getService(Components.interfaces.nsIIOService);
    var uri = ios.newURI("chrome://mediawrap/content/mediawrap.css", null, null);
    if (mediawrapUtils.getPref('mediawrapEnabled', true)
          && !sss.sheetRegistered(uri, sss.USER_SHEET)) {
      sss.loadAndRegisterSheet(uri, sss.USER_SHEET);
    }
    else if (!mediawrapUtils.getPref('mediawrapEnabled', true)
          && sss.sheetRegistered(uri, sss.USER_SHEET)) {
      sss.unregisterSheet(uri, sss.USER_SHEET);
    }

    this.updateStatusIcon();
  },

  getPref: function(prefname, value, unicode) {
    var prefService = Components.classes['@mozilla.org/preferences-service;1']
        .getService(Components.interfaces.nsIPrefService).getBranch('extensions.mediawrap.settings.');

    try {
      var scelta;
      if (typeof(value) == "boolean")
        scelta = prefService.getBoolPref(prefname);
      else if (typeof(value) == "number")
        scelta = prefService.getIntPref(prefname);
      else if (unicode && typeof(value) == "string")
        scelta = prefService.getComplexValue(prefname, Components.interfaces.nsISupportsString).data;
      else if (typeof(value) == "string")
        scelta = prefService.getCharPref(prefname);
      return scelta;
    } catch (e) {
      this.setPref(prefname, value, unicode);
      return value;
    }
  },

  setPref: function(prefname, value, unicode) {
    var prefService = Components.classes['@mozilla.org/preferences-service;1']
        .getService(Components.interfaces.nsIPrefService).getBranch('extensions.mediawrap.settings.');

    if (typeof(value) == "boolean")
      prefService.setBoolPref(prefname, value);
    else if (typeof(value) == "number")
      prefService.setIntPref(prefname, value);
    else if (unicode && typeof(value) == "string") {
      var str = Components.classes["@mozilla.org/supports-string;1"].
                createInstance(Components.interfaces.nsISupportsString);
      str.data = value;
      prefService.setComplexValue(prefname, Components.interfaces.nsISupportsString, str);
    }
    else if (typeof(value) == "string")
      prefService.setCharPref(prefname, value);
  },

  getEnabled: function() {
    return this.getPref('mediawrapEnabled', true);
  },

  setEnabled: function(bool) {
    this.setPref('mediawrapEnabled', bool);
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService);
    observerSvc.notifyObservers(null, "mediawrapStatusChanged", null);
  },

  fixMissingPluginBinding: function() {
    // in fx 3.0, embed target with unsupported type(missingPluginBinding.css registered in agent sheet)
    // can not be bounded
    var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"]
          .getService(Components.interfaces.nsIStyleSheetService);
    var ios = Components.classes["@mozilla.org/network/io-service;1"]
          .getService(Components.interfaces.nsIIOService);
    var uri = ios.newURI("chrome://mozapps/content/plugins/missingPluginBinding.css", null, null);
    if (sss.sheetRegistered(uri, sss.AGENT_SHEET)) {
      sss.unregisterSheet(uri, sss.AGENT_SHEET);
      // registered in user sheet again
      sss.loadAndRegisterSheet(uri, sss.USER_SHEET);
    }
  },

  onLoad: function() {

    try {
      window.removeEventListener("load", mediawrapUtils.onLoad, true);
    } catch (e) {}

    mediawrapUtils.fixMissingPluginBinding();
    mediawrapUtils.register();
    mediawrapUtils.statusChanged();
  },

  unLoad: function() {
    mediawrapUtils.unregister();
  }
}

window.addEventListener("load", mediawrapUtils.onLoad, false);
window.addEventListener("unload", mediawrapUtils.unLoad, false);
